using System;
using System.Collections.Generic;
using System.Text;
using System.ServiceModel;
using CompanyX.Insurance.La.DataContracts;
//using CompanyX.Insurance.La.FaultContracts;

namespace CompanyX.Insurance.La.ServiceContracts
{
    /// <summary>
    /// This is the interface definition for Life &amp; Annuity insurance common transactions
    /// </summary>
    /// <remarks>ACORD v2.18.02 compliant (includes custom schema extensions for operation faults)</remarks>
    [XmlSerializerFormat]
    [ServiceContract(Name = "Common", Namespace = "urn:CompanyX.Insurance.La.ServiceContracts")]
    public interface ICommon
    {
        /// <summary>
        /// This transaction is to request an illustration calculation engine to calculate and return values 
        /// for Illustration Purposes. If a Top-Level aggregate is not defined in this transaction, then it 
        /// cannot be used.
        /// </summary>
        /// <param name="request">TXLife Request Data</param>
        /// <returns>TXLife Response Data</returns>
        [OperationContract(Name = "CalculateForIllustration")]
        //[FaultContract(typeof(CalculateForIllustrationFt))]
        [return: MessageParameter(Name = "TXLifeResponse")]
        TXLifeResponse CalculateForIllustration([MessageParameter(Name = "TXLifeRequest")]TXLifeRequest request);

        /// <summary>
        /// This transaction is used to create, update, or delete Activity information. Allows any Activity object 
        /// to be updated using the standard DataRep values: "Full" to request an Insertion of new activity aggregate
        /// </summary>
        /// <param name="request">TXLife Request Data</param>
        /// <returns>TXLife Response Data</returns>
        [OperationContract(Name = "ProcessActivity")]
        //[FaultContract(typeof(ProcessActivityFt))]
        [return: MessageParameter(Name = "TXLifeResponse")]
        TXLifeResponse ProcessActivity([MessageParameter(Name = "TXLifeRequest")]TXLifeRequest request);

        /// <summary>
        /// The purpose of this transaction is to provide a generic mechanism to allow the sender to request that 
        /// XMLife data be added, updated, or deleted by the receiver. The "DataRep" attribute of the XMLife aggregates 
        /// is used to indicate which action is to be taken for each entity within the transaction.
        /// </summary>
        /// <param name="request">TXLife Request Data</param>
        /// <returns>TXLife Response Data</returns>
        [OperationContract(Name = "ProcessGenericDataTransfer")]
        //[FaultContract(typeof(ProcessGenericDataTransferFt))]
        [return: MessageParameter(Name = "TXLifeResponse")]
        TXLifeResponse ProcessGenericDataTransfer([MessageParameter(Name = "TXLifeRequest")]TXLifeRequest request);

        /// <summary>
        /// This transaction is used to return various policy values to a client.
        /// </summary>
        /// <param name="request">TXLife Request Data</param>
        /// <returns>TXLife Response Data</returns>
        [OperationContract(Name = "GetValues")]
        //[FaultContract(typeof(GetValuesFt))]
        [return: MessageParameter(Name = "TXLifeResponse")]
        TXLifeResponse GetValues([MessageParameter(Name = "TXLifeRequest")]TXLifeRequest request);

        /// <summary>
        /// This is the "push" version of the Values Inquiry message. It is used to transmit policy values. 
        /// When used in conjunction with TransSubTypeCode OLI_TRANSSUB_POSITIONS, it can be used to implement 
        /// a positions refresh message
        /// </summary>
        /// <param name="request">TXLife Request Data</param>
        /// <returns>TXLife Response Data</returns>
        [OperationContract(Name = "SendValues")]
        //[FaultContract(typeof(SendValuesFt))]
        [return: MessageParameter(Name = "TXLifeResponse")]
        TXLifeResponse SendValues([MessageParameter(Name = "TXLifeRequest")]TXLifeRequest request);

        /// <summary>
        /// Used to return activities to the requester.
        /// </summary>
        /// <param name="request">TXLife Request Data</param>
        /// <returns>TXLife Response Data</returns>
        [OperationContract(Name = "GetActivity")]
        //[FaultContract(typeof(GetActivityFt))]
        [return: MessageParameter(Name = "TXLifeResponse")]
        TXLifeResponse GetActivity([MessageParameter(Name = "TXLifeRequest")]TXLifeRequest request);

        /// <summary>
        /// The need to push an Activity to one or more Parties. This transaction is needed to push activity 
        /// information out. This will be used to transmit activity. This is not in response to an inquiry, but 
        /// rather the distribution of information so that receiver can take any required action that may be needed.
        /// </summary>
        /// <param name="request">TXLife Request Data</param>
        /// <returns>TXLife Response Data</returns>
        [OperationContract(Name = "SendActivity")]
        //[FaultContract(typeof(SendActivityFt))]
        [return: MessageParameter(Name = "TXLifeResponse")]
        TXLifeResponse SendActivity([MessageParameter(Name = "TXLifeRequest")]TXLifeRequest request);
    }
}
