using System;
using System.Collections.Generic;
using System.Text;
using System.ServiceModel;
using CompanyX.Insurance.La.DataContracts;
//using CompanyX.Insurance.La.FaultContracts;

namespace CompanyX.Insurance.La.ServiceContracts
{
    /// <summary>
    /// This is the interface definition for Life &amp; Annuity insurance financial transactions
    /// </summary>
    /// <remarks>ACORD v2.18.02 compliant (includes custom schema extensions for operation faults)</remarks>
    [XmlSerializerFormat]
    [ServiceContract(Name = "Financial", Namespace = "urn:CompanyX.Insurance.La.ServiceContracts")]
    public interface IFinancial
    {
        /// <summary>
        /// Used to do financial settlements between trading partners. If a Top-Level aggregate is not defined 
        /// in this transaction, then it cannot be used.
        /// </summary>
        /// <param name="request">TXLife Request Data</param>
        /// <returns>TXLife Response Data</returns>
        [OperationContract(Name = "Settlement")]
        //[FaultContract(typeof(SettlementFt))]
        [return: MessageParameter(Name = "TXLifeResponse")]
        TXLifeResponse Settlement([MessageParameter(Name = "TXLifeRequest")]TXLifeRequest request);

        /// <summary>
        /// Used is used to perform a bank routing validation license request. It is needed to validate an EFT 
        /// transaction. The response transaction would include a message that the routing numbers are valid or 
        /// invalid. An invalid routing number that can send information also assumes a status update as a 
        /// response transaction
        /// </summary>
        /// <param name="request">TXLife Request Data</param>
        /// <returns>TXLife Response Data</returns>
        [OperationContract(Name = "VaidateBankingRouting")]
        //[FaultContract(typeof(VaidateBankingRoutingFt))]
        [return: MessageParameter(Name = "TXLifeResponse")]
        TXLifeResponse VaidateBankingRouting([MessageParameter(Name = "TXLifeRequest")]TXLifeRequest request);

        /// <summary>
        /// To interface with a commission calculation subsystem (engine) to calculate commissions. Commission 
        /// events occur that need to determine commission payable. There may be many things that need 
        /// consideration in the calculation.
        /// </summary>
        /// <param name="request">TXLife Request Data</param>
        /// <returns>TXLife Response Data</returns>
        [OperationContract(Name = "CalculateCommission")]
        //[FaultContract(typeof(CalculateCommissionFt))]
        [return: MessageParameter(Name = "TXLifeResponse")]
        TXLifeResponse CalculateCommission([MessageParameter(Name = "TXLifeRequest")]TXLifeRequest request);

        /// <summary>
        /// This transaction is for the purpose of applying, reversing, refunding the various types of payments, 
        /// or to request these functions. This transaction is applicable to all product lines. The Arrangement 
        /// type or Financial Activity type is used to identify the type of payment that is being requested. The 
        /// use of Financial Activity or Arrangement is determined by their definition. 
        /// </summary>
        /// <param name="request">TXLife Request Data</param>
        /// <returns>TXLife Response Data</returns>
        [OperationContract(Name = "ProcessPayment")]
        //[FaultContract(typeof(ProcessPaymentFt))]
        [return: MessageParameter(Name = "TXLifeResponse")]
        TXLifeResponse ProcessPayment([MessageParameter(Name = "TXLifeRequest")]TXLifeRequest request);

        /// <summary>
        /// The purpose of this transaction is for quoting, processing, and reversing a policy loan. A loan is a 
        /// policy provision that grants the policy-owner the right to borrow part or all of the policy's net cash 
        /// value. This is different from a withdrawal in that a withdrawal reduces the policy's cash value and a 
        /// loan impairs the policy's cash value. The attached XML Data stream is an example of a loan on a variable 
        /// product. The policy-owner has specified the funds that he wants the loan to be processed against. The 
        /// policy-owner is overriding his standing tax withholding information and is not having taxes withheld. 
        /// Additionally, there is $100.00 in the POS department, and they want to return the money to the owner. 
        /// In addition to processing a loan for $500.00, $100.00 is being withdrawn from an account. The total 
        /// amount that the policy-owner will receive is $600.00.
        /// </summary>
        /// <param name="request">TXLife Request Data</param>
        /// <returns>TXLife Response Data</returns>
        [OperationContract(Name = "ProcessPolicyLoan")]
        //[FaultContract(typeof(ProcessPolicyLoanFt))]
        [return: MessageParameter(Name = "TXLifeResponse")]
        TXLifeResponse ProcessPolicyLoan([MessageParameter(Name = "TXLifeRequest")]TXLifeRequest request);

        /// <summary>
        /// To interface with a billing subsystem (engine), which assembles the various kinds of financial, 
        /// billing extracts to produce billing statements.
        /// </summary>
        /// <param name="request">TXLife Request Data</param>
        /// <returns>TXLife Response Data</returns>
        [OperationContract(Name = "AddBillingExtractStatement")]
        //[FaultContract(typeof(AddBillingExtractStatementFt))]
        [return: MessageParameter(Name = "TXLifeResponse")]
        TXLifeResponse AddBillingExtractStatement([MessageParameter(Name = "TXLifeRequest")]TXLifeRequest request);
    }
}
