using System;
using System.Collections.Generic;
using System.Text;
using System.ServiceModel;
using CompanyX.Insurance.La.DataContracts;
//using CompanyX.Insurance.La.FaultContracts;

namespace CompanyX.Insurance.La.ServiceContracts
{
    /// <summary>
    /// This is the interface definition for Life &amp; Annuity insurance investment transactions
    /// </summary>
    /// <remarks>ACORD v2.18.02 compliant (includes custom schema extensions for operation faults)</remarks>
    [XmlSerializerFormat]
    [ServiceContract(Name = "Investment", Namespace = "urn:CompanyX.Insurance.La.ServiceContracts")]
    public interface IInvestment
    {
        /// <summary>
        /// Used to change the standing allocations funds and percentages for which moneys (premiums) or charges 
        /// are applied.
        /// </summary>
        /// <param name="request">TXLife Request Data</param>
        /// <returns>TXLife Response Data</returns>
        [OperationContract(Name = "ChangeStandingAllocation")]
        //[FaultContract(typeof(ChangeStandingAllocationFt))]
        [return: MessageParameter(Name = "TXLifeResponse")]
        TXLifeResponse ChangeStandingAllocation([MessageParameter(Name = "TXLifeRequest")]TXLifeRequest request);

        /// <summary>
        /// Used to initiate a one-time transfer of money from one or more source fund(s) (Holding.Investment.SubAccount) to
        /// one or more destination fund(s). (For scheduling repetitive transfers, use Inforce Administration).
        /// </summary>
        /// <param name="request">TXLife Request Data</param>
        /// <returns>TXLife Response Data</returns>
        [OperationContract(Name = "TransferFunds")]
        //[FaultContract(typeof(TransferFundsFt))]
        [return: MessageParameter(Name = "TXLifeResponse")]
        TXLifeResponse TransferFunds([MessageParameter(Name = "TXLifeRequest")]TXLifeRequest request);

        /// <summary>
        /// Used to receive detailed product information for an investment product.
        /// </summary>
        /// <param name="request">TXLife Request Data</param>
        /// <returns>TXLife Response Data</returns>
        [OperationContract(Name = "GetInvestProductDetails")]
        //[FaultContract(typeof(GetInvestProductDetailsFt))]
        [return: MessageParameter(Name = "TXLifeResponse")]
        TXLifeResponse GetInvestProductDetails([MessageParameter(Name = "TXLifeRequest")]TXLifeRequest request);

        /// <summary>
        /// This transaction pushes the invest product information out. Used to transmit invest product information. 
        /// This is not in response to an inquiry but rather a distribution of information so that receiver can update 
        /// stored policy product information.
        /// </summary>
        /// <param name="request">TXLife Request Data</param>
        /// <returns>TXLife Response Data</returns>
        [OperationContract(Name = "SendInvestProductDetails")]
        //[FaultContract(typeof(SendInvestProductDetailsFt))]
        [return: MessageParameter(Name = "TXLifeResponse")]
        TXLifeResponse SendInvestProductDetails([MessageParameter(Name = "TXLifeRequest")]TXLifeRequest request);

        /// <summary>
        /// The search requests all use the CriteriaExpression child object of TXLifeRequest to express the search 
        /// criteria for the transaction. CriteriaExpression will support specifying explicit individual search 
        /// criteria (LastName = Smith) as well as complex logical expressions of individual criteria (LastName 
        /// = Smith AND ResidenceCounty = Northampton) grouped by the CriteriaOperator
        /// </summary>
        /// <param name="request">TXLife Request Data</param>
        /// <returns>TXLife Response Data</returns>
        [OperationContract(Name = "SearchInvestProducts")]
        //[FaultContract(typeof(SearchInvestProductsFt))]
        [return: MessageParameter(Name = "TXLifeResponse")]
        TXLifeResponse SearchInvestProducts([MessageParameter(Name = "TXLifeRequest")]TXLifeRequest request);
    }
}
