using System;
using System.Collections.Generic;
using System.Text;
using System.ServiceModel;
using CompanyX.Insurance.La.DataContracts;
//using CompanyX.Insurance.La.FaultContracts;

namespace CompanyX.Insurance.La.ServiceContracts
{
    /// <summary>
    /// This is the interface definition for Life &amp; Annuity insurance policy product
    /// </summary>
    /// <remarks>ACORD v2.18.02 compliant (includes custom schema extensions for operation faults)</remarks>
    [XmlSerializerFormat]
    [ServiceContract(Name = "PolicyProduct", Namespace = "urn:CompanyX.Insurance.La.ServiceContracts")]
    public interface IPolicyProduct
    {
        /// <summary>
        /// Used to request an inquiry for specific detailed product information for a policy product.
        /// </summary>
        /// <param name="request">TXLife Request Data</param>
        /// <returns>TXLife Response Data</returns>
        [OperationContract(Name = "GetProductPolicyDetails")]
        //[FaultContract(typeof(GetProductPolicyDetailsFt))]
        [return: MessageParameter(Name = "TXLifeResponse")]
        TXLifeResponse GetProductPolicyDetails([MessageParameter(Name = "TXLifeRequest")]TXLifeRequest request);

        /// <summary>
        /// This transaction pushes the policy product information out. Used for policy product downloads.
        /// </summary>
        /// <param name="request">TXLife Request Data</param>
        /// <returns>TXLife Response Data</returns>
        [OperationContract(Name = "SendProductPolicyDetails")]
        //[FaultContract(typeof(SendProductPolicyDetailsFt))]
        [return: MessageParameter(Name = "TXLifeResponse")]
        TXLifeResponse SendProductPolicyDetails([MessageParameter(Name = "TXLifeRequest")]TXLifeRequest request);

        /// <summary>
        /// Holding Product Inquiry supports the retrieval of detailed Product Information such as that returned in a 
        /// PolicyProduct Inquiry or an InvestProduct Inquiry, but identifies the information by its relationship to a 
        /// particular Holding. Holding Product Inquiry is not tied to a particular Product type, but will return the 
        /// appropriate Product structure based upon the Holding information and InquiryLevel specified
        /// </summary>
        /// <param name="request">TXLife Request Data</param>
        /// <returns>TXLife Response Data</returns>
        [OperationContract(Name = "GetProductPolicyDetailsForHolding")]
        //[FaultContract(typeof(GetProductPolicyDetailsForHoldingFt))]
        [return: MessageParameter(Name = "TXLifeResponse")]
        TXLifeResponse GetProductPolicyDetailsForHolding([MessageParameter(Name = "TXLifeRequest")]TXLifeRequest request);

        /// <summary>
        /// The search requests all use the CriteriaExpression child object of TXLifeRequest to express the search 
        /// criteria for the transaction. CriteriaExpression will support specifying explicit individual search 
        /// criteria (LastName = Smith) as well as complex logical expressions of individual criteria (LastName 
        /// = Smith AND ResidenceCounty = Northampton) grouped by the CriteriaOperator
        /// </summary>
        /// <param name="request">TXLife Request Data</param>
        /// <returns>TXLife Response Data</returns>
        [OperationContract(Name = "SearchProductPolicy")]
        //[FaultContract(typeof(SearchProductPolicyFt))]
        [return: MessageParameter(Name = "TXLifeResponse")]
        TXLifeResponse SearchProductPolicy([MessageParameter(Name = "TXLifeRequest")]TXLifeRequest request);
    }
}
